@@ -80,7 +80,7 @@ int only_blank(char *str)
    return 1;
 }
 
-static int blanks_marked ; /* song markers are positioned at start AND end of each song? */
+static int use_song_marker_pairs ; /* song markers are positioned at start AND end of each song? */
 
 #define ARRAYSIZE(x) (sizeof(x) / sizeof(x[0]))
 /* CD Tracks must be multiple of 588 or they will be padded with zeros */
@@ -163,7 +163,7 @@ void cdrdao_toc_info(char *filename)
     gtk_widget_show (song_table);
 
     row = 0;
-    for (j = 0; j < num_song_markers+1; j += 1+blanks_marked) {
+    for (j = 0; j < num_song_markers+1; j += 1+use_song_marker_pairs) {
        for (i = 0; i < ARRAYSIZE(song_info); i++) {
           char *init;
 
@@ -198,7 +198,7 @@ void cdrdao_toc_info(char *filename)
 	       if (strlen(album_info[i].init) == 0 && !only_blank(str))
 		  found_text = 1;;
 	   }
-	   for (j = 0; j < num_song_markers+(1-blanks_marked); j += 1+blanks_marked) {
+	   for (j = 0; j < num_song_markers+(1-use_song_marker_pairs); j += 1+use_song_marker_pairs) {
 	      for (i = 0; i < ARRAYSIZE(song_info); i++) {
 		  char *str;
 		  str = (char *)gtk_entry_get_text(GTK_ENTRY(song_widget[j][i]));
@@ -232,16 +232,32 @@ void cdrdao_toc_info(char *filename)
 	      new_cdtext = NULL;
 	   }
 
-	   for (j = 0; j < num_song_markers+(1-blanks_marked); j += 1+blanks_marked) {
+	   for (j = 0; j < num_song_markers+(1-use_song_marker_pairs); j += 1+use_song_marker_pairs) {
 	      long end;
-	      long start = song_markers[j] ;
-	      long length ;
 	      int j1 = j+1 ;
+	      long length ;
+	      long start ;
 
-	      if(j1 < num_song_markers) {
-		  length = song_markers[j+1] - start + 1 ;
+	      if(use_song_marker_pairs) {
+		  start = song_markers[j] ;
+
+		  if(j1 < num_song_markers) {
+		      length = song_markers[j+1] - start + 1 ;
+		  } else {
+		      /* no last song marker in last pair, assume end of audio for end of last track */
+		      length = (prefs.n_samples-1) - start + 1 ;
+		  }
 	      } else {
-		  length = (prefs.n_samples-1) - start + 1 ;
+		  if (j == 0) {
+		    start = 0 ;
+		    length = song_markers[j] - start + 1 ;
+		  } else if (j == num_song_markers) {
+		    start = song_markers[j-1] ;
+		    length = (prefs.n_samples-1) - start + 1 ;
+		  } else {
+		    start = song_markers[j-1] ;
+		    length = song_markers[j] - start + 1 ;
+		  }
 	      }
 
 #define AUDIO_BLOCK_LEN 588
@@ -249,6 +265,12 @@ void cdrdao_toc_info(char *filename)
 
 	      end = start + length -1 ;
 
+	      while(end > (prefs.n_samples-1) && end > 2*AUDIO_BLOCK_LEN) {
+		end -= AUDIO_BLOCK_LEN ;
+	      }
+
+	      length = end - start + 1 ;
+
 	      fprintf(toc, "TRACK AUDIO\n");
 	      if (found_text) {
 		 fprintf(toc, "   CD_TEXT {\n      LANGUAGE 0 {\n");
@@ -353,13 +375,13 @@ void save_cdrdao_toc(GtkWidget * widget, gpointer data)
 
 void save_cdrdao_tocs(GtkWidget * widget, gpointer data)
 {
-    blanks_marked = 0 ;
+    use_song_marker_pairs = 0 ;
     save_cdrdao_toc(widget, data) ;
 }
 
 void save_cdrdao_tocp(GtkWidget * widget, gpointer data)
 {
-    blanks_marked = 1 ;
+    use_song_marker_pairs = 1 ;
     save_cdrdao_toc(widget, data) ;
 }
 
