@@ -683,7 +683,7 @@ void set_misc_preferences(GtkWidget * widget, gpointer data)
 
     audio_device_entry =
 	add_number_entry_with_label(audio_device,
-			   "Audio device (try /dev/dsp for OSS, hw:0,0 or hw 1,0 ... for ALSA", dialog_table, row++);
+			   "Audio device try (/dev/dsp for OSS) (default, hw:0,0 or hw:1,0 ... for ALSA)", dialog_table, row++);
 
 
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), dialog_table,
